//=============================================================================
// ScriptedTextureController.
//=============================================================================
class ScriptedTextureController expands ClientScriptedTextureInfo;

enum BounceType
	{
		BT_Horiz,
		BT_Vert,
		BT_Diag
	};
var(BounceSettings) BounceType BounceDir;


struct HZBoundries
{
	var() int LeftBound;
	var() int RightBound;
	var() int DeltaX;
};
var(BounceSettings) HZBoundries HorizBounds;

struct VTBoundries
{
	var() int TopBound;
	var() int BottomBound;
	var() int DeltaY;
};
var(BounceSettings) VTBoundries VerticalBounds;

//------------------------------------------------------------------------------------------
// Normal vars
var() color TextColor;
var() int X;
var() int Y;
var() Font DrawFont;
var(Strings) String DisplayString[8];
var(Strings) String AltString[8];
var(Strings) int StringMargin; // The distance between each string (y-value)
var() bool bColored;
var() bool bTriggerChangeString;
var() bool bTriggeredOnceOnly;
var string initialstring[8];

//------------------------------------------------------------------------------------------
// Non-state based functions
simulated function BeginPlay()
{
local int r;
Super.BeginPlay();
	if (x == 0)
		{
			x = 32;
		}
	if (y == 0)
		{
			y = 64; 
		}
	if (DrawFont == None)
		{
			DrawFont = Font'WhiteFont';
		}
}

simulated function PostBeginPlay()
{
local int q;
		for (q = 0; q < 8; q++)
		{
			initialstring[q] = displaystring[q];	// make sure that the initial string setting is saved
		}
		
		if (StringMargin == 0 || StringMargin < 8) // make sure the user was not an idiot and put 0 or something dumb there
			StringMargin = 8;
}

function Trigger(actor Other, pawn EventInstigator)
{
local int e; 
		
		if (bTriggerChangeString)
		{
			for(e = 0; e < 8; e++)
			{	
				if (DisplayString[e] == InitialString[e])
					DisplayString[e] = AltString[e];
				else if (DisplayString[e] == AltString[e] && bTriggeredOnceOnly == False)
					DisplayString[e] = InitialString[e];
			}
		}
}

simulated function RenderTexture(scriptedtexture Tex)
{
	// Debug
}

//------------------------------------------------------------------------------------------
// Draw one line of colored text

State() DrawColorText
{
	simulated event RenderTexture(ScriptedTexture Tex)
	{	
		tex.DrawColoredText(X,Y,displaystring[0],DrawFont,TextColor);
	}
}

//------------------------------------------------------------------------------------------
// Text bounces around

state() BouncingText
{
	simulated event rendertexture(Scriptedtexture tex)
	{
		switch(BounceDir)
		{
			case BT_Horiz:
			
			x += HorizBounds.DeltaX; 
						
			if (x > HorizBounds.RightBound || x < HorizBounds.LeftBound) 
			{
				HorizBounds.DeltaX *= -1;					
			}
			
			if(bColored)
			{
				tex.DrawColoredText(X,Y,displaystring[0],DrawFont,TextColor);
			}
			else
			{
				tex.DrawText(X,Y,displaystring[0],DrawFont);
			}
			break;
			
			
			
			case BT_Vert:
			
			Y += VerticalBounds.DeltaY;
			
			if (y < VerticalBounds.BottomBound || y > VerticalBounds.TopBound)
			{
				VerticalBounds.DeltaY *= -1;
			}
			
			if(bColored)
			{
				tex.DrawColoredText(X,Y,displaystring[0],DrawFont,TextColor);
			}
			else
			{
				tex.DrawText(X,Y,displaystring[0],DrawFont);
			}
			break;
			
			
			case BT_Diag: 
			
			Y += VerticalBounds.DeltaY; 
			x += HorizBounds.DeltaX; 	
				
			if (x > HorizBounds.RightBound || x < HorizBounds.LeftBound)
			{
				HorizBounds.DeltaX *= -1;
			}
			else
			{}
			
			if (y < VerticalBounds.BottomBound || y > VerticalBounds.TopBound)
			{
				VerticalBounds.DeltaY *= -1;
			}
			else 
			{}
			
			if(bColored)
			{
				tex.DrawColoredText(X,Y,displaystring[0],DrawFont,TextColor);
			}
			else
			{
				tex.DrawText(X,Y,displaystring[0],DrawFont);
			}
			break;
			
			default:	
		}	
	}
}

//------------------------------------------------------------------------------------------------
// Multiple string handling

state() MultipleStrings
{	
	simulated function RenderTexture(ScriptedTexture Tex)
	{
		local int p; 
		
		if (bColored)
		{
			for (p = 0; p < 8; p++)
			{
				if (DisplayString[p] != "") 
				tex.DrawColoredText(X,Y + (StringMargin * p),DisplayString[p],DrawFont,TextColor);
			}
		}
		else
		{
			for (p = 0; p < 8; p++)
			{
				if (DisplayString[p] != "") // If the string is available
				tex.DrawText(X,Y + (StringMargin * p),DisplayString[p],DrawFont); // Display it
			}
		}	
	}
}

replication
{
	reliable if (remoterole == role_dumbproxy)
		x,y,drawfont,textcolor,displaystring,bcolored,horizbounds,verticalbounds,bouncedir,altstring;
}

//-------------------------------------------------------------------------------------------------
// Scrolling string

state() ScrollingString
{
	simulated function RenderTexture(scriptedtexture Tex)
	{
		X += HorizBounds.Deltax;
		
		if (x == HorizBounds.RightBound)
			x = HorizBounds.LeftBound;
			
		else if (x == HorizBounds.LeftBound)
			x = HorizBounds.RightBound;
			
		if(bColored)
		{
			tex.DrawColoredText(X,Y,displaystring[0],DrawFont,TextColor);
		}
		else
		{
			tex.DrawText(X,Y,displaystring[0],DrawFont);
		}
	}
}

defaultproperties
{
				HorizBounds=(LeftBound=-256,RightBound=512,DeltaX=2)
				TextColor=(R=27,G=147,B=205)
				X=-254
				Y=40
				DrawFont=Font'Engine.MedFont'
				DisplayString(0)="This is a test!"
				StringMargin=16
				bColored=True
				ScriptedTexture=None
				RemoteRole=ROLE_DumbProxy
				InitialState="ScrollingString"
}
